Module: HappyMapper::ClassMethods

Defined in:
lib/happymapper.rb

Instance Method Summary collapse

Instance Method Details

#attribute(name, type, options = {}) ⇒ Object



23
24
25
26
27
28
# File 'lib/happymapper.rb', line 23

def attribute(name, type, options={})
  attribute = Attribute.new(name, type, options)
  @attributes[to_s] ||= []
  @attributes[to_s] << attribute
  attr_accessor attribute.method_name.intern
end

#attributesObject



30
31
32
# File 'lib/happymapper.rb', line 30

def attributes
  @attributes[to_s] || []
end

#element(name, type, options = {}) ⇒ Object



34
35
36
37
38
39
# File 'lib/happymapper.rb', line 34

def element(name, type, options={})
  element = Element.new(name, type, options)
  @elements[to_s] ||= []
  @elements[to_s] << element
  attr_accessor element.method_name.intern
end

#elementsObject



41
42
43
# File 'lib/happymapper.rb', line 41

def elements
  @elements[to_s] || []
end

#has_many(name, type, options = {}) ⇒ Object



49
50
51
# File 'lib/happymapper.rb', line 49

def has_many(name, type, options={})
  element name, type, {:single => false}.merge(options)
end

#has_one(name, type, options = {}) ⇒ Object



45
46
47
# File 'lib/happymapper.rb', line 45

def has_one(name, type, options={})
  element name, type, {:single => true}.merge(options)
end

#namespace(namespace = nil) ⇒ Object

Specify a namespace if a node and all its children are all namespaced elements. This is simpler than passing the :namespace option to each defined element.



56
57
58
59
# File 'lib/happymapper.rb', line 56

def namespace(namespace = nil)
  @namespace = namespace if namespace
  @namespace
end

#parse(xml, options = {}) ⇒ Object



69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
# File 'lib/happymapper.rb', line 69

def parse(xml, options = {})
  # locally scoped copy of namespace for this parse run
  namespace = @namespace

  if xml.is_a?(XML::Node)
    node = xml
  else
    if xml.is_a?(XML::Document)
      node = xml.root
    else
      node = XML::Parser.string(xml).parse.root
    end

    root = node.name == tag_name
  end

  # This is the entry point into the parsing pipeline, so the default
  # namespace prefix registered here will propagate down
  namespaces = node.namespaces
  if namespaces && namespaces.default
    # don't assign the default_prefix if it has already been assigned
    namespaces.default_prefix = DEFAULT_NS unless namespaces.find_by_prefix(DEFAULT_NS)
    namespace ||= DEFAULT_NS
  end

  xpath = root ? '/' : './/'
  xpath += "#{namespace}:" if namespace
  #puts "parse: #{xpath}"

  nodes = []
  # when finding nodes, do it in this order:
  # 1. specified tag
  # 2. name of element
  # 3. tag_name (derived from class name by default)
  [options[:tag], options[:name], tag_name].compact.each do |xpath_ext|
    nodes = node.find(xpath + xpath_ext.to_s)
    break if nodes && nodes.size > 0
  end
  collection = nodes.collect do |n|
    obj = new

    attributes.each do |attr|
      obj.send("#{attr.method_name}=",
                attr.from_xml_node(n, namespace))
    end

    elements.each do |elem|
      obj.send("#{elem.method_name}=",
                elem.from_xml_node(n, namespace))
    end

    obj
  end

  # per http://libxml.rubyforge.org/rdoc/classes/LibXML/XML/Document.html#M000354
  nodes = nil

  if options[:single] || root
    collection.first
  else
    collection
  end
end

#tag(new_tag_name) ⇒ Object



61
62
63
# File 'lib/happymapper.rb', line 61

def tag(new_tag_name)
  @tag_name = new_tag_name.to_s unless new_tag_name.nil? || new_tag_name.to_s.empty?
end

#tag_nameObject



65
66
67
# File 'lib/happymapper.rb', line 65

def tag_name
  @tag_name ||= to_s.split('::')[-1].downcase
end